# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import netCDF4
import pandas as pd
import xarray as xr
import cartopy.crs as ccrs
import matplotlib as mpl
import matplotlib.pyplot as plt
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import datetime

####################################################################

title = [ '(a) GLB100km', '(b) NP10km', '(c) CMEMS' ]

styr=1993
edyr=2018
styr_cmems=1993
edyr_cmems=2018

suptitle = 'EKE (' + str(styr) + '-' + str(edyr) + ' mean)'
outfile = './fig/EKE_comp_'+str(styr) + '-' + str(edyr)

####################################################################
# CMEMS

print( "Loading CMEMS data" )

path_ref='../data/refdata/CMEMS/'
basename_ref = 'hs_eke_uv'
suffix = '.' + '{:04}_{:04}'.format(styr_cmems,edyr_cmems)
eke_np = path_ref + basename_ref + suffix

nx=1440
ny=720
fl = open(eke_np,'rb')
eke_tmp = np.fromfile(fl, dtype = '>f4', count= nx*ny)
eke_cmems=eke_tmp.astype(np.float64).reshape(ny,nx).copy()
eke_cmems=np.where(eke_cmems > 9990, np.NaN, eke_cmems*10000)


lat = np.array(np.empty((ny)),dtype=np.float64)
for j in range(ny):
    lat[j] = -89.875 + 0.25 * j

lon = np.array(np.empty((nx)),dtype=np.float64)
for i in range(nx):
    lon[i] = 0.125 + 0.25 * i

DS_cmems = xr.Dataset( {'eke': (['lat','lon'], eke_cmems),},
                       coords = { 'lat': lat, 'lon': lon, } )


####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_glb100km='../data/glb100km/'
path_g100=path_glb100km+'run-20211104o2/ANALY/annclim/'
basename_glb = 'hs_eke.anncl'
suffix = '.' + '{:04}-{:04}.nc'.format(styr,edyr)
eke_glb = path_g100 + basename_glb + suffix

DS1 = xr.open_dataset( eke_glb )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_np10km='../data/np10km/'
path_np=path_np10km+'hst_annclim-np/'
basename_np = 'hs_eke.anncl'
suffix = '.' + '{:04}-{:04}.nc'.format(styr,edyr)
eke_np = path_np + basename_np + suffix

DS0 = xr.open_dataset( eke_np )

################################

print( 'Start drawing' )
fig = plt.figure(figsize=(8.5,11))
fig.suptitle( suptitle, fontsize=18 )

proj = ccrs.PlateCarree(central_longitude=-140.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()

ax = [ plt.subplot(3,1,1,projection=proj),
       plt.subplot(3,1,2,projection=proj),
       plt.subplot(3,1,3,projection=proj) ]

bounds = [0, 1, 2, 5, 10, 20, 100, 200, 500, 1000, 2000]
ticks_bounds = [0, 1, 2, 5, 10, 20, 100, 200, 500, 1000, 2000] 


da0=DS1['eke'].sel(lon=slice(110,285)).sel(lat=slice(-15,65)).isel(depth=4).isel(time=0)*0.5
da0.where(da0>0.0).plot(ax=ax[0],cmap='plasma',
                            levels=bounds,
                            extend='both',
                            cbar_kwargs = { 'extend': 'max',
                                            'orientation': 'vertical',
                                            'spacing': 'uniform',
                                            'label': r'$\mathrm{cm}^2\,\mathrm{s}^{-2}$',
                                            'shrink': 0.8,
                                            'ticks': ticks_bounds, },
                            transform=ccrs.PlateCarree())

ax[0].coastlines()
ax[0].set_xticks(np.arange(120,285,30),crs=ccrs.PlateCarree())
ax[0].set_yticks(np.arange(-10,60.1,10),crs=ccrs.PlateCarree())
ax[0].set_extent([110,285,-15,65],crs=ccrs.PlateCarree())
ax[0].xaxis.set_major_formatter(lon_formatter)
ax[0].yaxis.set_major_formatter(lat_formatter)
ax[0].set_aspect(1.2)
ax[0].set_xlabel('longitude',fontsize=14)
ax[0].set_ylabel('latitude',fontsize=14)
ax[0].title.set_size(16)
ax[0].set_title(title[0],{'fontsize':12, 'verticalalignment':'top', 'linespacing':1.0})
ax[0].tick_params(labelsize=12)
#ax.background_patch.set_facecolor('lightgray')
ax[0].patch.set_facecolor('lightgray')


da1=DS0['eke'].sel(lon=slice(110,285)).sel(lat=slice(-15,65)).isel(depth=4).isel(time=0)*0.5
da1.where(da1>0.0).plot(ax=ax[1],cmap='plasma',
                            levels=bounds,
                            extend='both',
                            cbar_kwargs = { 'extend': 'max',
                                            'orientation': 'vertical',
                                            'spacing': 'uniform',
                                            'label': r'$\mathrm{cm}^2\,\mathrm{s}^{-2}$',
                                            'shrink': 0.8,
                                            'ticks': ticks_bounds, },
                            transform=ccrs.PlateCarree())

ax[1].coastlines()
ax[1].set_xticks(np.arange(120,285,30),crs=ccrs.PlateCarree())
ax[1].set_yticks(np.arange(-10,60.1,10),crs=ccrs.PlateCarree())
ax[1].set_extent([110,285,-15,65],crs=ccrs.PlateCarree())
ax[1].xaxis.set_major_formatter(lon_formatter)
ax[1].yaxis.set_major_formatter(lat_formatter)
ax[1].set_aspect(1.2)
ax[1].set_xlabel('longitude',fontsize=14)
ax[1].set_ylabel('latitude',fontsize=14)
ax[1].title.set_size(16)
ax[1].set_title(title[1],{'fontsize':12, 'verticalalignment':'top', 'linespacing':1.0})
ax[1].tick_params(labelsize=12)
#ax.background_patch.set_facecolor('lightgray')
ax[1].patch.set_facecolor('lightgray')

##########

da2=DS_cmems['eke'].sel(lon=slice(110,285)).sel(lat=slice(-15,65))
da2.where(da2>0.0).plot(ax=ax[2],cmap='plasma',
                            levels=bounds,
                            extend='both',
                            cbar_kwargs = { 'extend': 'max',
                                            'orientation': 'vertical',
                                            'spacing': 'uniform',
                                            'label': r'$\mathrm{cm}^2\,\mathrm{s}^{-2}$',
                                            'shrink': 0.8,
                                            'ticks': ticks_bounds, },
                            transform=ccrs.PlateCarree())

ax[2].coastlines()
ax[2].set_xticks(np.arange(120,285,30),crs=ccrs.PlateCarree())
ax[2].set_yticks(np.arange(-10,60.1,10),crs=ccrs.PlateCarree())
ax[2].set_extent([110,285,-15,65],crs=ccrs.PlateCarree())
ax[2].xaxis.set_major_formatter(lon_formatter)
ax[2].yaxis.set_major_formatter(lat_formatter)
ax[2].set_aspect(1.2)
ax[2].set_xlabel('longitude',fontsize=14)
ax[2].set_ylabel('latitude',fontsize=14)
ax[2].title.set_size(16)
ax[2].set_title(title[2],{'fontsize':12, 'verticalalignment':'top', 'linespacing':1.0})
ax[2].tick_params(labelsize=12)
#ax.background_patch.set_facecolor('lightgray')
ax[2].patch.set_facecolor('lightgray')


plt.subplots_adjust(left=0.10,right=0.98,bottom=0.12,top=0.92,hspace=0.32,wspace=0.15)

outpdf = outfile + '.pdf'
outpng = outfile + '.png'
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.0)
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.0)
print("figures are saved to " + outpng + " and " + outpdf)

plt.show()
